@@ -4,15 +4,14 @@ const Clutter = imports.gi.Clutter;
 const GLib = imports.gi.GLib;
 const St = imports.gi.St;
 const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
 const DND = imports.ui.dnd;
 const Tooltips = imports.ui.tooltips;
 const PopupMenu = imports.ui.popupMenu;
 const Mainloop = imports.mainloop;
 const {SignalManager} = imports.misc.signalManager;
 const {unref} = imports.misc.util;
-const {createStore} = imports.misc.state;
 
+const createStore = require('./state');
 const {AppMenuButtonRightClickMenu, HoverMenuController, AppThumbnailHoverMenu} = require('./menus');
 const {
     FLASH_INTERVAL,
@@ -63,7 +62,7 @@ const getFocusState = function(metaWindow) {
 class AppGroup {
     constructor(params) {
         this.state = params.state;
-        this.listState = params.listState;
+        this.workspaceState = params.workspaceState;
         this.groupState = createStore({
             app: params.app,
             appId: params.appId,
@@ -255,10 +254,10 @@ class AppGroup {
     }
 
     setMargin() {
-        const applet = this.state.appletActor;
+        const appletActor = this.state.appletActor;
         const direction = this.state.isHorizontal ? 'right' : 'bottom';
         const existingStyle = this.actor.style ? this.actor.style : '';
-        let spacing = parseInt(applet.get_theme_node().get_length('spacing'));
+        let spacing = parseInt(appletActor.get_theme_node().get_length('spacing'));
         if (!spacing) {
             spacing = 6;
         }
@@ -347,7 +346,7 @@ class AppGroup {
 
         const allocateForLabel = this.labelVisiblePref ||
                             (this.state.settings.titleDisplay == TitleDisplay.Focused &&
-                            this.listState.lastFocusedApp === appId);
+                            this.workspaceState.lastFocusedApp === appId);
 
         if (this.state.orientation === St.Side.TOP || this.state.orientation === St.Side.BOTTOM) {
             if (allocateForLabel) {
@@ -473,11 +472,11 @@ class AppGroup {
             return;
         }
 
-        Tweener.addTween(this.label, {
+        this.label.ease({
             width,
-            time: BUTTON_BOX_ANIMATION_TIME,
-            transition: 'easeOutQuad',
-            onComplete: () => {
+            duration: BUTTON_BOX_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onStopped: () => {
                 if (!this.label) return;
                 this.label.show();
             }
@@ -582,13 +581,13 @@ class AppGroup {
         const {appId, metaWindows, lastFocused} = this.groupState;
 
         if (hasFocus === undefined) {
-            hasFocus = this.listState.lastFocusedApp === appId;
+            hasFocus = this.workspaceState.lastFocusedApp === appId;
         }
 
         // If any of the windows associated with our app have focus,
         // we should set ourselves to active
         if (hasFocus) {
-            this.listState.trigger('updateFocusState', appId);
+            this.workspaceState.trigger('updateFocusState', appId);
             this.actor.add_style_pseudo_class('focus');
             this.actor.remove_style_class_name('grouped-window-list-item-demands-attention');
             if (this.hoverMenu) {
@@ -610,9 +609,9 @@ class AppGroup {
         if (!this.groupState || !this.groupState.groupReady || this.groupState.willUnmount) {
             return;
         }
-        const windows = this.groupState.metaWindows;
-        for (let i = 0, len = windows.length; i < len; i++) {
-            if (windows[i] === metaWindow) {
+        
+        this.groupState.metaWindows.forEach( window => {
+            if (window === metaWindow && !getFocusState(window)) {
                 // Even though this may not be the last focused window, we want it to be
                 // the window that gets focused when a user responds to an alert.
                 this.groupState.set({lastFocused: metaWindow});
@@ -620,7 +619,7 @@ class AppGroup {
                 this.getAttention();
                 return true;
             }
-        }
+        });
         return false;
     }
 
@@ -655,7 +654,7 @@ class AppGroup {
                 Main.activateWindow(this.groupState.lastFocused, global.get_current_time());
             } else {
                 if (this.groupState.fileDrag) {
-                    this.listState.trigger('closeAllHoverMenus');
+                    this.workspaceState.trigger('closeAllHoverMenus');
                 }
                 // Open the thumbnail menu and activate the window corresponding to the dragged over thumbnail.
                 if (!this.hoverMenu) this.initThumbnailMenu();
@@ -801,13 +800,13 @@ class AppGroup {
         } else if (button === 3) {
             if (!this.rightClickMenu) this.initRightClickMenu();
             if (!this.rightClickMenu.isOpen) {
-                this.listState.trigger('closeAllRightClickMenus', () => {
-                    this.listState.trigger('closeAllHoverMenus', () => {
+                this.workspaceState.trigger('closeAllRightClickMenus', () => {
+                    this.workspaceState.trigger('closeAllHoverMenus', () => {
                         this.rightClickMenu.toggle();
                     });
                 });
             } else {
-                this.listState.trigger('closeAllRightClickMenus', this.listState.trigger('closeAllHoverMenus'));
+                this.workspaceState.trigger('closeAllRightClickMenus', this.workspaceState.trigger('closeAllHoverMenus'));
             }
         }
         if (this.hoverMenu) this.hoverMenu.onButtonPress();
@@ -834,7 +833,7 @@ class AppGroup {
                 if (!this.hoverMenu) this.initThumbnailMenu();
                 this.hoverMenu.open(true);
             } else {
-                this.listState.trigger('closeAllHoverMenus');
+                this.workspaceState.trigger('closeAllHoverMenus');
             }
             this.windowHandle();
             this.appKeyTimeout = setTimeout(() => {
@@ -949,7 +948,7 @@ class AppGroup {
             this.calcWindowNumber();
         } else {
             // This is the last window, so this group needs to be destroyed. We'll call back windowRemoved
-            // in appList to put the final nail in the coffin.
+            // in workspace to put the final nail in the coffin.
             if (typeof cb === 'function') {
                 if (this.hoverMenu && this.groupState.isFavoriteApp) {
                     this.groupState.trigger('removeThumbnailFromMenu', metaWindow);
@@ -960,10 +959,10 @@ class AppGroup {
     }
 
     onAppChange(metaWindow) {
-        if (!this.listState) return;
+        if (!this.workspaceState) return;
 
-        this.listState.trigger('windowRemoved', metaWindow);
-        this.listState.trigger('windowAdded', metaWindow);
+        this.workspaceState.trigger('windowRemoved', metaWindow);
+        this.workspaceState.trigger('windowAdded', metaWindow);
     }
 
     onWindowTitleChanged(metaWindow, refresh) {
@@ -1015,7 +1014,7 @@ class AppGroup {
 
         const hasFocus = getFocusState(metaWindow);
         if (hasFocus && this.groupState.hasOwnProperty('lastFocused')) {
-            this.listState.set({lastFocusedApp: this.groupState.appId});
+            this.workspaceState.set({lastFocusedApp: this.groupState.appId});
             this.groupState.set({lastFocused: metaWindow});
         }
         this.onFocusChange(hasFocus);
@@ -1059,7 +1058,7 @@ class AppGroup {
         if (this.actor.is_finalized()) return;
 
         if (changed) {
-            setTimeout(() => this.listState.trigger('updateAppGroupIndexes', this.groupState.appId), 0);
+            setTimeout(() => this.workspaceState.trigger('updateAppGroupIndexes', this.groupState.appId), 0);
         }
 
         if (this.groupState.metaWindows.length === 0 && this.state.appletReady) {
@@ -1104,32 +1103,39 @@ class AppGroup {
         if (effect === 1) return;
         else if (effect === 2) {
             this.iconBox.set_z_rotation_from_gravity(0.0, Clutter.Gravity.CENTER);
-            Tweener.addTween(this.iconBox, {
+            this.iconBox.ease({
                 opacity: 70,
-                time: 0.2,
-                transition: 'linear',
-                onCompleteScope: this,
-                onComplete() {
-                    Tweener.addTween(this.iconBox, {
+                duration: 200,
+                mode: Clutter.AnimationMode.LINEAR,
+                onStopped: () => {
+                   this.iconBox.ease({
                         opacity: 255,
-                        time: 0.2,
-                        transition: 'linear'
+                        duration: 200,
+                        mode: Clutter.AnimationMode.LINEAR,
                     });
                 }
             });
         } else if (effect === 3) {
             this.iconBox.set_pivot_point(0.5, 0.5);
-            Tweener.addTween(this.iconBox, {
+            this.iconBox.ease({
                 scale_x: 0.8,
                 scale_y: 0.8,
-                time: 0.2,
-                transition: 'easeOutQuad',
-                onComplete: () => {
-                    Tweener.addTween(this.iconBox, {
-                        scale_x: 1.0,
-                        scale_y: 1.0,
-                        time: 0.2,
-                        transition: 'easeOutQuad'
+                duration: 175,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                onStopped: () => {
+                    this.iconBox.ease({
+                        scale_x: 1.1,
+                        scale_y: 1.1,
+                        duration: 175,
+                        mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                        onStopped: () => {
+                            this.iconBox.ease({
+                                scale_x: 1.0,
+                                scale_y: 1.0,
+                                duration: 50,
+                                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+                            });
+                        }
                     });
                 }
             });
@@ -1157,7 +1163,7 @@ class AppGroup {
             this.hoverMenu.destroy();
         }
 
-        this.listState.trigger('removeChild', this.actor);
+        this.workspaceState.trigger('removeChild', this.actor);
         this.actor.destroy();
 
         if (!skipRefCleanup) {
