@@ -2,15 +2,14 @@ const Clutter = imports.gi.Clutter;
 const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Gio = imports.gi.Gio;
-const AppletManager = imports.ui.appletManager;
 const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
 const PopupMenu = imports.ui.popupMenu;
 const Applet = imports.ui.applet;
 const SignalManager = imports.misc.signalManager;
 const WindowUtils = imports.misc.windowUtils;
+const Mainloop = imports.mainloop;
 
-const {each, findIndex, tryFn, unref, trySpawnCommandLine, spawn_async, getDesktopActionIcon} = imports.misc.util;
+const {tryFn, unref, trySpawnCommandLine, spawn_async, getDesktopActionIcon} = imports.misc.util;
 const {
     CLOSE_BTN_SIZE,
     CLOSED_BUTTON_STYLE,
@@ -27,17 +26,17 @@ const convertRange = function(value, r1, r2) {
 const setOpacity = (peekTime, window_actor, targetOpacity, cb) => {
     const opacity = convertRange(targetOpacity, [0, 100], [0, 255]);
 
-    const tweenConfig = {
-        time: peekTime * 0.001,
-        transition: 'easeOutQuad',
-        opacity: opacity > 255 ? 255 : opacity
+    const easeConfig = {
+        duration: peekTime,
+        mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+        opacity: opacity > 255 ? 255 : opacity,
     };
 
     if (typeof cb === 'function') {
-        tweenConfig.onComplete = cb;
+        easeConfig.onStopped = cb;
     }
 
-    Tweener.addTween(window_actor, tweenConfig);
+    window_actor.ease(easeConfig);
 };
 
 class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
@@ -52,9 +51,9 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
     monitorMoveWindows(i) {
         if (this.state.settings.monitorMoveAllWindows) {
-            let metaWindows = this.groupState.metaWindows.slice();
+            const metaWindows = this.groupState.metaWindows.slice();
             while (metaWindows.length > 0) {
-                let metaWindow = metaWindows[0];
+                const metaWindow = metaWindows[0];
                 if (metaWindow === this.groupState.lastFocused) {
                     Main.activateWindow(metaWindow, global.get_current_time());
                 }
@@ -73,10 +72,10 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
         let item;
         let length;
-        let hasWindows = this.groupState.metaWindows.length > 0;
-        let isWindowBacked = this.groupState.app.is_window_backed();
+        const hasWindows = this.groupState.metaWindows.length > 0;
+        const isWindowBacked = this.groupState.app.is_window_backed();
 
-        let createMenuItem = (opts = {label: '', icon: null}) => {
+        const createMenuItem = (opts = {label: '', icon: null}) => {
             if (opts.icon) {
                 return new PopupMenu.PopupIconMenuItem(opts.label, opts.icon, St.IconType.SYMBOLIC);
             }
@@ -90,7 +89,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         if (hasWindows) {
             // Monitors
             if (Main.layoutManager.monitors.length > 1) {
-                let connectMonitorEvent = (item, i) => {
+                const connectMonitorEvent = (item, i) => {
                     this.signals.connect(item, 'activate', () => this.monitorMoveWindows(i));
                 };
                 for (let i = 0, len = Main.layoutManager.monitors.length; i < len; i++) {
@@ -129,17 +128,17 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                     item = new PopupMenu.PopupSubMenuMenuItem(_('Move to another workspace'));
                     this.addMenuItem(item);
 
-                    let connectWorkspaceEvent = (ws, j) => {
+                    const connectWorkspaceEvent = (ws, j) => {
                         this.signals.connect(ws, 'activate', () => {
                             this.groupState.lastFocused.change_workspace(global.workspace_manager.get_workspace_by_index(j));
                         });
                     };
                     for (let i = 0; i < length; i++) {
                         // Make the index a local variable to pass to function
-                        let j = i;
-                        let name = Main.workspace_names[i] ? Main.workspace_names[i] : Main._makeDefaultWorkspaceName(i);
-                        let menuItem = createMenuItem({label: _(name)});
-                        let ws = this.groupState.lastFocused.get_workspace();
+                        const j = i;
+                        const name = Main.workspace_names[i] ? Main.workspace_names[i] : Main._makeDefaultWorkspaceName(i);
+                        const menuItem = createMenuItem({label: _(name)});
+                        const ws = this.groupState.lastFocused.get_workspace();
 
                         if (ws && i === ws.index()) {
                             menuItem.setSensitive(false);
@@ -154,7 +153,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         }
 
         // Preferences
-        let subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Applet preferences'));
+        const subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Applet preferences'));
         this.addMenuItem(subMenu);
 
         item = createMenuItem({label: _('About...'), icon: 'dialog-question'});
@@ -174,14 +173,14 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         if (this.state.settings.showRecent) {
             // Places
             if (this.groupState.appId === 'nemo.desktop' || this.groupState.appId === 'nemo-home.desktop') {
-                let subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Places'));
+                const subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Places'));
                 this.addMenuItem(subMenu);
 
-                let defualtPlaces = this.listDefaultPlaces();
-                let bookmarks = this.listBookmarks();
-                let devices = this.listDevices();
-                let places = [...defualtPlaces, ...bookmarks, ...devices];
-                let handlePlaceLaunch = (item, i) => {
+                const defualtPlaces = this.listDefaultPlaces();
+                const bookmarks = this.listBookmarks();
+                const devices = this.listDevices();
+                const places = [...defualtPlaces, ...bookmarks, ...devices];
+                const handlePlaceLaunch = (item, i) => {
                     this.signals.connect(item, 'activate', () => places[i].launch());
                 };
                 for (let i = 0, len = places.length; i < len; i++) {
@@ -193,12 +192,12 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             }
 
             // Recent Files
-            let recentItems = this.state.trigger('getRecentItems');
-            let items = [];
+            const recentItems = this.state.trigger('getRecentItems');
+            const items = [];
 
             for (let i = 0, len = recentItems.length; i < len; i++) {
-                let mimeType = recentItems[i].get_mime_type();
-                let appInfo = Gio.app_info_get_default_for_type(mimeType, false);
+                const mimeType = recentItems[i].get_mime_type();
+                const appInfo = Gio.app_info_get_default_for_type(mimeType, false);
                 if (appInfo && this.groupState.appInfo && appInfo.get_id() === this.groupState.appId) {
                     items.push(recentItems[i]);
                 }
@@ -206,13 +205,13 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             let itemsLength = items.length;
 
             if (itemsLength > 0) {
-                let subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Recent'));
+                const subMenu = new PopupMenu.PopupSubMenuMenuItem(_('Recent'));
                 this.addMenuItem(subMenu);
-                let num = 10;
+                const num = 10;
                 if (itemsLength > num) {
                     itemsLength = num;
                 }
-                let handleRecentLaunch = (item, i) => {
+                const handleRecentLaunch = (item, i) => {
                     this.signals.connect(item, 'activate', () => {
                         Gio.app_info_launch_default_for_uri(items[i].get_uri(), global.create_app_launch_context())
                     });
@@ -227,7 +226,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         }
 
         if (Main.gpu_offload_supported && !hasWindows) {
-            item = createMenuItem({label: _("Run with NVIDIA GPU"), icon: 'cpu'});
+            item = createMenuItem({label: _("Run with dedicated GPU"), icon: 'cpu'});
 
             this.signals.connect(item, 'activate', () => this.groupState.trigger('launchNewInstance', true));
 
@@ -238,10 +237,10 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
         // Actions
         tryFn(() => {
             if (!this.groupState.appInfo) return;
-            let actions = this.groupState.appInfo.list_actions();
+            const actions = this.groupState.appInfo.list_actions();
             if (actions) {
                 this.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
-                let handleAction = (action) => {
+                const handleAction = (action) => {
                     let icon = getDesktopActionIcon(action);
                     if (icon == null)
                         icon = 'application-x-executable';
@@ -282,7 +281,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             this.signals.connect(this.pinToggleItem, 'activate', (...args) => this.toggleFavorite(...args));
             this.addMenuItem(this.pinToggleItem);
             if (this.state.settings.autoStart) {
-                let label = this.groupState.autoStartIndex !== -1 ? _('Remove from Autostart') : _('Add to Autostart');
+                const label = this.groupState.autoStartIndex !== -1 ? _('Remove from Autostart') : _('Add to Autostart');
                 item = createMenuItem({label: label, icon: 'insert-object'});
                 this.signals.connect(item, 'activate', (...args) => this.toggleAutostart(...args));
                 this.addMenuItem(item);
@@ -297,7 +296,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
 
         // Window controls
         if (hasWindows) {
-            let metaWindowActor = this.groupState.lastFocused.get_compositor_private();
+            const metaWindowActor = this.groupState.lastFocused.get_compositor_private();
             // Miscellaneous
             if (metaWindowActor && metaWindowActor.opacity !== 255) {
                 item = createMenuItem({label: _('Restore to full opacity')});
@@ -338,7 +337,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
                 // Close others
                 item = createMenuItem({label: _('Close others'), icon: 'window-close'});
                 this.signals.connect(item, 'activate', () => {
-                    each(this.groupState.metaWindows, (metaWindow) => {
+                    this.groupState.metaWindows.forEach( metaWindow => {
                         if (metaWindow !== this.groupState.lastFocused && !metaWindow._needsAttention) {
                             metaWindow.delete(global.get_current_time());
                         }
@@ -389,7 +388,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             this.state.autoStartApps.splice(this.groupState.autoStartIndex, 1);
             this.groupState.set({autoStartIndex: -1});
         } else {
-            let filePath = this.groupState.appInfo.get_filename();
+            const filePath = this.groupState.appInfo.get_filename();
             trySpawnCommandLine('bash -c "cp ' + filePath + ' ' + autoStartStrDir + '"');
             setTimeout(() => {
                 this.state.trigger('getAutoStartApps');
@@ -411,8 +410,8 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
     }
 
     createShortcut() {
-        let proc = this.groupState.lastFocused.get_pid();
-        let cmd = [
+        const proc = this.groupState.lastFocused.get_pid();
+        const cmd = [
             'bash',
             '-c',
             'python3 /usr/share/cinnamon/applets/grouped-window-list@cinnamon.org/utils.py get_process ' + proc.toString()
@@ -421,15 +420,15 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             if (stdout) {
                 setTimeout(() => {
                     this.state.trigger('addFavorite', {appId: stdout.trim(), app: null, pos: -1});
-                    this.state.trigger('refreshCurrentAppList');
+                    this.state.trigger('refreshCurrentWorkspace');
                 }, 2000);
             }
         });
     }
 
     listDefaultPlaces(pattern) {
-        let defaultPlaces = Main.placesManager.getDefaultPlaces();
-        let res = [];
+        const defaultPlaces = Main.placesManager.getDefaultPlaces();
+        const res = [];
         for (let i = 0, len = defaultPlaces.length; i < len; i++) {
             if (!pattern || defaultPlaces[i].name.toLowerCase().indexOf(pattern) !== -1) {
                 res.push(defaultPlaces[i]);
@@ -439,8 +438,8 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
     }
 
     listBookmarks(pattern) {
-        let bookmarks = Main.placesManager.getBookmarks();
-        let res = [];
+        const bookmarks = Main.placesManager.getBookmarks();
+        const res = [];
         for (let i = 0, len = bookmarks.length; i < len; i++) {
             if (!pattern || bookmarks[i].name.toLowerCase().indexOf(pattern) !== -1) {
                 res.push(bookmarks[i]);
@@ -450,8 +449,8 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
     }
 
     listDevices(pattern) {
-        let devices = Main.placesManager.getMounts();
-        let res = [];
+        const devices = Main.placesManager.getMounts();
+        const res = [];
         for (let i = 0, len = devices.length; i < len; i++) {
             if (!pattern || devices[i].name.toLowerCase().indexOf(pattern) !== -1) {
                 res.push(devices[i]);
@@ -534,6 +533,8 @@ class WindowThumbnail {
         this.metaWindow = params.metaWindow;
         this.index = params.index;
 
+        this.get_thumb_id = 0;
+
         this.metaWindowActor = null;
         this.thumbnailPadding = 16;
         this.willUnmount = false;
@@ -566,7 +567,7 @@ class WindowThumbnail {
             y_expand: false
         });
 
-        let label = new St.Label({
+        const label = new St.Label({
             style_class: 'grouped-window-list-thumbnail-label',
             important: true
         });
@@ -627,16 +628,21 @@ class WindowThumbnail {
         this.button.set_opacity(0);
     }
 
-    onWindowDemandsAttention(window) {
-        if (this._needsAttention) {
-            return false;
-        }
-        this._needsAttention = true;
-        if (this.metaWindow === window) {
+    setThumbnailDemandsAttention(attention) {
+        //if (this.metaWindow === window) {
+        if (attention) {
+            if (this._needsAttention) {
+                return;
+            }
+            this._needsAttention = true;
             this.actor.add_style_class_name('grouped-window-list-thumbnail-alert');
-            return true;
+        } else {
+            if (!this._needsAttention) {
+                return;
+            }
+            this._needsAttention = false;
+            this.actor.remove_style_class_name('grouped-window-list-thumbnail-alert');
         }
-        return false;
     }
 
     onFocusWindowChange() {
@@ -672,7 +678,7 @@ class WindowThumbnail {
     }
 
     onCloseButtonRelease(actor, event) {
-        let button = event.get_button();
+        const button = event.get_button();
         if (button === 1 && actor === this.button) {
             this.handleCloseClick();
         }
@@ -683,7 +689,7 @@ class WindowThumbnail {
             this.groupState.trigger('hoverMenuClose');
             return;
         }
-        let button = typeof event === 'number' ? event : event.get_button();
+        const button = typeof event === 'number' ? event : event.get_button();
         if (button === 1 && !this.stopClick) {
             Main.activateWindow(this.metaWindow, global.get_current_time());
             this.groupState.trigger('hoverMenuClose');
@@ -710,7 +716,7 @@ class WindowThumbnail {
         if (this.metaWindowActor && !this.metaWindowActor.is_finalized()) {
             this.signals.connect(this.metaWindow, 'unmanaging', () => this.disconnectSizeNotify());
 
-            let texture = this.metaWindowActor.get_texture();
+            const texture = this.metaWindowActor.get_texture();
             if (texture == null) {
                 return;
             }
@@ -718,7 +724,7 @@ class WindowThumbnail {
             this.signals.connect(texture, 'size-changed', () => this.refreshThumbnail());
 
             let [width, height] = this.metaWindowActor.get_size();
-            let scale = Math.min(1.0, thumbnailWidth / width, thumbnailHeight / height) * global.ui_scale;
+            const scale = Math.min(1.0, thumbnailWidth / width, thumbnailHeight / height) * global.ui_scale;
             width = Math.round(width * scale);
             height = Math.round(height * scale);
             if (this.thumbnailActor.child == null || (this.thumbnailActor.child.name?.startsWith("TextureWindowClone"))) {
@@ -745,13 +751,20 @@ class WindowThumbnail {
     disconnectSizeNotify(actor) {
         this.signals.disconnect('unmanaging', this.metaWindow);
 
-        let texture = this.metaWindowActor.get_texture();
+        const texture = this.metaWindowActor.get_texture();
         if (texture) {
             this.signals.disconnect("size-changed", texture);
         }
         this.signals.disconnect("notify::minimized", this.metaWindow);
     }
 
+    calcThumbsFullSize(singleThumbSize) {
+        const padding = this.thumbnailActor.style_length('padding');
+        const margin = this.thumbnailActor.style_length('margin');
+        const size = (singleThumbSize + this.thumbnailPadding) * global.ui_scale;
+        return (size + padding + margin) * this.groupState.windowCount;
+    }
+
     refreshThumbnail() {
         if (this.willUnmount
             || !this.groupState
@@ -761,12 +774,12 @@ class WindowThumbnail {
             return;
         }
 
-        let monitor = this.state.trigger('getPanelMonitor');
+        const monitor = this.state.trigger('getPanelMonitor');
         if (!monitor) return;
 
         if (!this.thumbnailActor || this.thumbnailActor.is_finalized()) return;
 
-        let divider = 80 * global.ui_scale;
+        const divider = 80 * global.ui_scale;
         let {thumbSize} = this.state.settings;
 
         if (monitor.height / global.ui_scale <= 1024) {
@@ -787,16 +800,15 @@ class WindowThumbnail {
             thumbnailSize = thumbnailWidth;
         }
 
-        let padding = this.thumbnailActor.style_length('padding');
-        let margin = this.thumbnailActor.style_length('margin');
 
         let i = 0;
-        while (((thumbnailSize + this.thumbnailPadding + padding + margin) * this.groupState.windowCount > monitorSize)
-            && thumbnailWidth > 64
-            && thumbnailHeight > 64) {
+        while (this.calcThumbsFullSize(thumbnailSize) > monitorSize &&
+               thumbnailWidth > 64 && thumbnailHeight > 64) {
+            // ---
             thumbnailWidth -= 1;
             thumbnailHeight -= 1;
             thumbnailSize -= 1;
+
             i++;
             // Bail after 200 iterations
             if (i > 200) {
@@ -806,22 +818,32 @@ class WindowThumbnail {
 
         // If we can't fit all the thumbnails, revert to a vertical menu orientation
         // with no thumbnails, which can hold more window selections.
-        let verticalThumbs = ((thumbnailSize + this.thumbnailPadding + padding + margin) * this.groupState.windowCount) > monitorSize;
-        let currentVerticalThumbsState = this.groupState.verticalThumbs;
+        const verticalThumbs = this.calcThumbsFullSize(thumbnailSize) > monitorSize;
+        const currentVerticalThumbsState = this.groupState.verticalThumbs;
         this.groupState.set({verticalThumbs});
+
         if (verticalThumbs !== currentVerticalThumbsState) return;
 
-        let scaledWidth = thumbnailWidth * global.ui_scale;
-        this.thumbnailActor.width = scaledWidth;
+        this.thumbnailActor.width = thumbnailWidth * global.ui_scale;
         this.container.style = `width: ${Math.floor(thumbnailWidth - 16)}px;`;
-        if (this.groupState.verticalThumbs || (this.state.settings.verticalThumbs && this.state.settings.showThumbs)) {
-            this.thumbnailActor.height = thumbnailHeight;
-        } else if (this.state.settings.verticalThumbs) {
+
+        if (this.state.settings.verticalThumbs && !this.groupState.verticalThumbs && !this.state.settings.showThumbs) {
             this.thumbnailActor.height = 0;
+        } else {
+            this.thumbnailActor.height = thumbnailHeight * global.ui_scale;
         }
 
         this.labelContainer.child.text = this.metaWindow.title || '';
-        this.getThumbnail(thumbnailWidth, thumbnailHeight);
+
+        if (this.get_thumb_id > 0) {
+            Mainloop.source_remove(this.get_thumb_id);
+        }
+
+        this.get_thumb_id = Mainloop.timeout_add(0, () => {
+            this.get_thumb_id = 0;
+            this.getThumbnail(thumbnailWidth, thumbnailHeight);
+            return false;
+        });
     }
 
     hoverPeek(opacity) {
@@ -851,7 +873,7 @@ class WindowThumbnail {
         if (!this.state.lastOverlayPreview) return;
 
         if (this.state.settings.peekTimeOut) {
-            let currOverlayPreview = this.state.lastOverlayPreview;
+            const currOverlayPreview = this.state.lastOverlayPreview;
             setOpacity(
                 this.state.settings.peekTimeOut,
                 currOverlayPreview,
@@ -876,6 +898,11 @@ class WindowThumbnail {
         this.willUnmount = true;
         if (!this.groupState) return;
 
+        if (this.get_thumb_id > 0) {
+            Mainloop.source_remove(this.get_thumb_id);
+            this.get_thumb_id = 0;
+        }
+
         this.state.disconnect(this.stateConnectId);
         this.groupState.disconnect(this.connectId);
         this.signals.disconnectAllSignals();
@@ -908,8 +935,8 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
                 // over the menu and closes it if not.
                 setTimeout(() => {
                     let [x, y, mask] = global.get_pointer();
-                    let draggedOverActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
-                    let parent = draggedOverActor.get_parent();
+                    const draggedOverActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
+                    const parent = draggedOverActor.get_parent();
                     if (!(parent instanceof St.Widget)) {
                         this.close(true);
                     }
@@ -924,7 +951,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
                 this.queuedWindows.push(win);
             },
             removeThumbnailFromMenu: (win) => {
-                let index = findIndex(this.appThumbnails, (item) => item.metaWindow === win);
+                let index = this.appThumbnails.findIndex( item => item.metaWindow === win);
                 if (index > -1) {
                     this.appThumbnails[index].destroy();
                     this.appThumbnails[index] = undefined;
@@ -935,7 +962,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
             },
             verticalThumbs: () => {
                 // Preserve the menu's open state after refreshing
-                let {isOpen} = this;
+                const {isOpen} = this;
                 this.setVerticalSetting();
                 if (isOpen) this.open(true);
             },
@@ -946,18 +973,18 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
                     // 50ms loop until the menu closes so we continue getting data in the absence of events.
                     this.interval = setInterval(() => {
                         let [x, y, mask] = global.get_pointer();
-                        let draggedOverActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
+                        const draggedOverActor = global.stage.get_actor_at_pos(Clutter.PickMode.ALL, x, y);
                         if (draggedOverActor instanceof Meta.ShapedTexture) {
                             this.groupState.set({fileDrag: false});
                             this.close(true);
                             return;
                         }
-                        each(this.appThumbnails, function(thumbnail) {
+                        for (const thumbnail of this.appThumbnails) {
                             if (thumbnail.thumbnailActor === draggedOverActor) {
                                 Main.activateWindow(thumbnail.metaWindow, global.get_current_time());
-                                return false;
+                                break;
                             }
-                        });
+                        }
                     }, 50);
                 } else if (this.interval) {
                     clearInterval(this.interval);
@@ -972,7 +999,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
 
     addQueuedThumbnails() {
         if (this.queuedWindows.length === 0) return;
-        each(this.queuedWindows, (win) => this.addThumbnail(win));
+        this.queuedWindows.forEach( win => this.addThumbnail(win));
         this.queuedWindows = [];
     }
 
@@ -1024,7 +1051,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
     }
 
     onKeyRelease(actor, event) {
-        let symbol = event.get_key_symbol();
+        const symbol = event.get_key_symbol();
         if (this.isOpen && (symbol === Clutter.KEY_Super_L || symbol === Clutter.KEY_Super_R)) {
             // Close this menu, if opened by super + #
             this.close(true);
@@ -1075,16 +1102,14 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
     }
 
     onKeyPress(actor, e) {
-        let {orientation} = this.state;
-        let {vertical} = this.box;
+        const {orientation} = this.state;
+        const {vertical} = this.box;
 
-        let symbol = e.get_key_symbol();
-        let i = findIndex(this.appThumbnails, (item) => item.entered === true);
-        let entered = i > -1;
+        const symbol = e.get_key_symbol();
+        let i = this.appThumbnails.findIndex( item => item.entered === true );
+        const entered = i > -1;
         if (!entered) {
-            i = findIndex(this.appThumbnails, function(thumbnail) {
-                return thumbnail.isFocused;
-            });
+            i = this.appThumbnails.findIndex( thumbnail => thumbnail.isFocused );
             if (i === -1) {
                 i = 0;
             }
@@ -1177,9 +1202,9 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
                 return b.metaWindow.user_time - a.metaWindow.user_time;
             });
         }
-        let refThumb = findIndex(this.appThumbnails, (thumbnail) => thumbnail.metaWindow === metaWindow);
+        const refThumb = this.appThumbnails.findIndex( thumbnail => thumbnail.metaWindow === metaWindow );
         if (!this.appThumbnails[refThumb] && refThumb === -1) {
-            let thumbnail = new WindowThumbnail({
+            const thumbnail = new WindowThumbnail({
                 state: this.state,
                 groupState: this.groupState,
                 metaWindow: metaWindow,
@@ -1237,7 +1262,7 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
         this.willUnmount = true;
         if (!this.box) return;
 
-        if (this.isOpen) this.close();
+        if (this.isOpen) this.close(true);
 
         for (let w = 0, len = this.appThumbnails.length; w < len; w++) {
             if (this.appThumbnails[w] !== undefined) {
@@ -1255,3 +1280,9 @@ class AppThumbnailHoverMenu extends PopupMenu.PopupMenu {
         unref(this, RESERVE_KEYS);
     }
 }
+
+module.exports = {
+    AppMenuButtonRightClickMenu,
+    HoverMenuController,
+    AppThumbnailHoverMenu
+};
