@@ -182,7 +182,7 @@ class CsdDBusProxyHandler(object):
         try:
             Gio.DBusProxy.new_for_bus(Gio.BusType.SYSTEM, Gio.DBusProxyFlags.NONE, None,
                                       'org.cinnamon.SettingsDaemon.DateTimeMechanism',
-                                      '/',
+                                      '/org/cinnamon/SettingsDaemon/DateTimeMechanism',
                                       'org.cinnamon.SettingsDaemon.DateTimeMechanism',
                                       None, self._on_proxy_ready, None)
         except dbus.exceptions.DBusException as e:
@@ -255,7 +255,7 @@ class TimeZoneSelector(SettingsWidget):
             self.region_map[region].append([city, _(city_display_name)])
 
     def set_timezone(self, timezone):
-        if timezone == "Etc/UTC":
+        if timezone == "Etc/UTC" or timezone == "Universal":
             return
 
         self.timezone = timezone
