@@ -235,7 +235,7 @@ class MnemonicLabel(Gtk.Label):
         self.set_text_with_mnemonic(text)
         self.set_mnemonic_widget(widget)
         self.set_alignment(0.0, 0.5)
-        self.set_valign(Gtk.Align.START)
+        self.set_valign(Gtk.Align.CENTER)
         self.set_line_wrap(True)
 
 class DefaultAppChooserButton(Gtk.AppChooserButton):
@@ -247,6 +247,14 @@ class DefaultAppChooserButton(Gtk.AppChooserButton):
         self.set_show_dialog_item(True)
         self.connect("changed", self.onChanged)
 
+    def do_scroll_event(self, event, data=None):
+        # Skip Gtk.ComboBox's default handler.
+        #
+        # Connecting to a Gtk.ComboBox and stopping a scroll-event
+        # prevents unintentional combobox changes, but also breaks
+        # any scrollable parents when passing over the combobox.
+        Gtk.Widget.do_scroll_event(self, event)
+
     def onChanged(self, button):
         info = button.get_app_info()
 
@@ -306,13 +314,21 @@ class DefaultTerminalButton(Gtk.AppChooserButton):
             # terminals don't have mime types, so we check for "TerminalEmulator" under the "Category" key in desktop files
             if cat_val is not None and "TerminalEmulator" in cat_val:
                 # this crazy if statement makes sure remaining desktop file info is not empty, then prevents root terminals from showing, then prevents repeating terminals from trying to being added which leave a blank space and Gtk-WARNING's
-                if exec_val is not None and name_val is not None and icon_val is not None and not "gksu" in exec_val and exec_val not in self.active_items:
+                if exec_val is not None and name_val is not None and icon_val is not None and "gksu" not in exec_val and exec_val not in self.active_items:
                     self.append_custom_item(exec_val, name_val, Gio.ThemedIcon.new(icon_val))
                     self.active_items.append(exec_val)
                     if self.key_value == exec_val:
                         self.set_active_custom_item(self.key_value)
             count_up += 1
 
+    def do_scroll_event(self, event, data=None):
+        # Skip Gtk.ComboBox's default handler.
+        #
+        # Connecting to a Gtk.ComboBox and stopping a scroll-event
+        # prevents unintentional combobox changes, but also breaks
+        # any scrollable parents when passing over the combobox.
+        Gtk.Widget.do_scroll_event(self, event)
+
     def onChanged(self, button):
         index_num = button.get_active()
         command_key = self.active_items[index_num]
