@@ -118,7 +118,7 @@ const CinnamonIface =
             </method> \
             <signal name="MonitorsChanged"/> \
             <method name="GetRunState"> \
-               <arg type="i" direction="out" name="state" /> \
+                <arg type="i" direction="out" name="state" /> \
             </method> \
             <method name="RestartCinnamon"> \
                 <arg type="b" direction="in" name="show_osd" /> \
@@ -127,6 +127,14 @@ const CinnamonIface =
             <signal name="RunStateChanged"/> \
             <signal name="XletsLoadedComplete"/> \
             <property name="AnimationsEnabled" type="b" access="read" /> \
+            <method name="ShowMonitorLabels"> \
+                <arg type="a{sv}" direction="in" name="params"/> \
+            </method> \
+            <method name="HideMonitorLabels"/> \
+            <method name="ShowEndSessionDialog"> \
+                <arg type="i" direction="in" name="mode"/> \
+            </method> \
+            <method name="CloseEndSessionDialog"/> \
         </interface> \
     </node>';
 
@@ -254,7 +262,7 @@ CinnamonDBus.prototype = {
         if (params['icon'])
             icon = Gio.Icon.new_for_string(params['icon']);
 
-        Main.osdWindowManager.show(monitorIndex, icon, params['level'], false);
+        Main.osdWindowManager.show(monitorIndex, icon, params['label'], params['level'], false);
     },
 
     FlashArea: function(x, y, width, height) {
@@ -499,5 +507,28 @@ CinnamonDBus.prototype = {
         this._dbusImpl.emit_property_changed('AnimationsEnabled', variant);
     },
 
+    ShowMonitorLabelsAsync(monitor_info, invocation) {
+        if (Main.monitorLabeler != null) {
+            Main.monitorLabeler.show(monitor_info[0], invocation.get_sender());
+        }
+    },
+
+    HideMonitorLabelsAsync(tuple, invocation) {
+        if (Main.monitorLabeler != null) {
+            Main.monitorLabeler.hide(invocation.get_sender());
+        }
+    },
+
+    ShowEndSessionDialog(mode) {
+        GLib.idle_add(GLib.PRIORITY_DEFAULT, () => {
+            Main.showEndSessionDialog(mode);
+            return GLib.SOURCE_REMOVE;
+        });
+    },
+
+    CloseEndSessionDialog() {
+        Main.closeEndSessionDialog();
+    },
+
     CinnamonVersion: Config.PACKAGE_VERSION
 };
