@@ -1,11 +1,11 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
-const Lang = imports.lang;
 const St = imports.gi.St;
 const Atk = imports.gi.Atk;
 const Cinnamon = imports.gi.Cinnamon;
 const Signals = imports.signals;
+const GObject = imports.gi.GObject;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
 const Pango = imports.gi.Pango;
@@ -15,16 +15,14 @@ const Gdk = imports.gi.Gdk;
 const Params = imports.misc.params;
 const Util = imports.misc.util;
 
+const Dialog = imports.ui.dialog;
+const Layout = imports.ui.layout;
 const Lightbox = imports.ui.lightbox;
 const Main = imports.ui.main;
-const Tweener = imports.ui.tweener;
 
 const Gettext = imports.gettext;
 
-const FADE_IN_BUTTONS_TIME = 0.33;
-const FADE_OUT_DIALOG_TIME = 1.0;
-
-var OPEN_AND_CLOSE_TIME = 0.1;
+const FADE_OUT_DIALOG_TIME = 1000;
 
 var State = {
     OPENED: 0,
@@ -49,11 +47,17 @@ var State = {
  * For simple usage such as displaying a message, or asking for confirmation,
  * the #ConfirmDialog and #NotifyDialog classes may be used instead.
  */
-function ModalDialog(params) {
-    this._init(params);
-}
-
-ModalDialog.prototype = {
+var ModalDialog = GObject.registerClass({
+    Properties: {
+        'state': GObject.ParamSpec.int(
+            'state', 'Dialog state', 'state',
+            GObject.ParamFlags.READABLE,
+            Math.min(...Object.values(State)),
+            Math.max(...Object.values(State)),
+            State.CLOSED)
+    },
+    Signals: { 'opened': {}, 'closed': {} }
+}, class ModalDialog extends St.Widget {
     /**
      * _init:
      * @params (JSON): parameters for the modal dialog. Options include
@@ -61,82 +65,85 @@ ModalDialog.prototype = {
      * block Cinnamon input, and @styleClass, which is the style class the
      * modal dialog should use.
      */
-    _init: function(params) {
-        params = Params.parse(params, { cinnamonReactive: false,
-                                        styleClass: null });
+    _init(params) {
+        super._init({
+            visible: false,
+            x: 0,
+            y: 0,
+            accessible_role: Atk.Role.DIALOG,
+        });
 
-        this.state = State.CLOSED;
+        params = Params.parse(params, {
+            cinnamonReactive: Main.virtualKeyboard.enabled,
+            styleClass: null,
+            destroyOnClose: true,
+        });
+
+        this._state = State.CLOSED;
         this._hasModal = false;
         this._cinnamonReactive = params.cinnamonReactive;
+        this._destroyOnClose = params.destroyOnClose;
 
-        this._group = new St.Widget({ visible: false,
-                                      x: 0,
-                                      y: 0,
-                                      accessible_role: Atk.Role.DIALOG });
-        Main.uiGroup.add_actor(this._group);
+        Main.uiGroup.add_actor(this);
 
-        let constraint = new Clutter.BindConstraint({ source: global.stage,
-                                                      coordinate: Clutter.BindCoordinate.POSITION | Clutter.BindCoordinate.SIZE });
-        this._group.add_constraint(constraint);
+        let constraint = new Clutter.BindConstraint({
+            source: global.stage,
+            coordinate: Clutter.BindCoordinate.POSITION | Clutter.BindCoordinate.SIZE
+        });
+        this.add_constraint(constraint);
 
-        this._group.connect('destroy', Lang.bind(this, this._onGroupDestroy));
+        this.backgroundStack = new St.Widget({ layout_manager: new Clutter.BinLayout() });
+        this._backgroundBin = new St.Bin({
+            child: this.backgroundStack,
+            x_fill: true,
+            y_fill: true
+        });
+        this._monitorConstraint = new Layout.MonitorConstraint({ work_area: true });
+        this._backgroundBin.add_constraint(this._monitorConstraint);
+        this.add_actor(this._backgroundBin);
 
-        this._actionKeys = {};
-        this._group.connect('key-press-event', Lang.bind(this, this._onKeyPressEvent));
+        this.dialogLayout = new Dialog.Dialog(this.backgroundStack, params.styleClass);
+        this.contentLayout = this.dialogLayout.contentLayout;
+        this.buttonLayout = this.dialogLayout.buttonLayout;
 
-        this._backgroundBin = new St.Bin();
-        this._group.add_actor(this._backgroundBin);
-
-        this._dialogLayout = new St.BoxLayout({ style_class: 'modal-dialog',
-                                                vertical:    true });
-        if (params.styleClass != null) {
-            this._dialogLayout.add_style_class_name(params.styleClass);
+        this.openAndCloseTime = 100;
+        let enableRadialEffect = true;
+        if (!global.settings.get_boolean("desktop-effects-workspace")) {
+            this.openAndCloseTime = 0;
+            enableRadialEffect = false;
         }
 
         if (!this._cinnamonReactive) {
-            this._lightbox = new Lightbox.Lightbox(this._group,
+            this._lightbox = new Lightbox.Lightbox(this,
                                                    { inhibitEvents: true,
-                                                     radialEffect: true });
+                                                     radialEffect: enableRadialEffect });
             this._lightbox.highlight(this._backgroundBin);
 
-            let stack = new Cinnamon.Stack();
-            this._backgroundBin.child = stack;
-
-            this._eventBlocker = new Clutter.Group({ reactive: true });
-            stack.add_actor(this._eventBlocker);
-            stack.add_actor(this._dialogLayout);
-        } else {
-            this._backgroundBin.child = this._dialogLayout;
+            this._eventBlocker = new Clutter.Actor({ reactive: true });
+            this.backgroundStack.add_actor(this._eventBlocker);
         }
 
-
-        this.contentLayout = new St.BoxLayout({ vertical: true });
-        this._dialogLayout.add(this.contentLayout,
-                               { x_fill:  true,
-                                 y_fill:  true,
-                                 x_align: St.Align.MIDDLE,
-                                 y_align: St.Align.START });
-
-        this._buttonLayout = new St.BoxLayout({ style_class: 'modal-dialog-button-box',
-                                                vertical:    false });
-        this._dialogLayout.add(this._buttonLayout,
-                               { expand:  true,
-                                 x_align: St.Align.MIDDLE,
-                                 y_align: St.Align.END });
-
-        global.focus_manager.add_group(this._dialogLayout);
-        this._initialKeyFocus = this._dialogLayout;
+        global.focus_manager.add_group(this.dialogLayout);
+        this._initialKeyFocus = null;
+        this._initialKeyFocusDestroyId = 0;
         this._savedKeyFocus = null;
-    },
+    }
 
-    /**
-     * destroy:
-     *
-     * Destroys the modal dialog
-     */
-    destroy: function() {
-        this._group.destroy();
-    },
+    get state() {
+        return this._state;
+    }
+
+    _setState(state) {
+        if (this._state == state)
+            return;
+
+        this._state = state;
+        this.notify('state');
+    }
+
+    clearButtons() {
+        this.dialogLayout.clearButtons();
+    }
 
     /**
      * setButtons:
@@ -152,146 +159,70 @@ ModalDialog.prototype = {
      * Optional arguments include @focused, which determines whether the button
      * is initially focused, and @key, which is a keybinding associated with
      * the button press such that pressing the keybinding will have the same
-     * effect as clicking the button.
+     * effect as clicking the button. Other arguments include @default
+     * and @destructive_action which add additional styling.
      *
      * An example usage is
      * ```
      * dialog.setButtons([
      *     {
      *         label: _("Cancel"),
-     *         action: Lang.bind(this, this.callback),
+     *         action: this.callback.bind(this),
      *         key: Clutter.KEY_Escape
      *     },
      *     {
      *         label: _("OK"),
-     *         action: Lang.bind(this, this.destroy),
-     *         key: Clutter.KEY_Return
+     *         action: this.destroy.bind(this),
+     *         key: Clutter.KEY_Return,
+     *         default: true
      *     }
      * ]);
      * ```
      */
-    setButtons: function(buttons) {
-        let hadChildren = this._buttonLayout.get_n_children() > 0;
+    setButtons(buttons) {
+        this.clearButtons();
 
-        this._buttonLayout.destroy_all_children();
-        this._actionKeys = {};
-        let focusSetExplicitly = false;
-
-        for (let i = 0; i < buttons.length; i ++) {
-            let buttonInfo = buttons[i];
-            if (!buttonInfo.focused) {
-                buttonInfo.focused = false;
-            }
-            let label = buttonInfo['label'];
-            let action = buttonInfo['action'];
-            let key = buttonInfo['key'];
-            let wantsfocus = buttonInfo['focused'] === true;
-            let nofocus = buttonInfo['focused'] === false;
-            buttonInfo.button = new St.Button({ style_class: 'modal-dialog-button',
-                                                reactive:    true,
-                                                can_focus:   true,
-                                                label:       label });
-
-            let x_alignment;
-            if (buttons.length == 1)
-                x_alignment = St.Align.END;
-            else if (i == 0)
-                x_alignment = St.Align.START;
-            else if (i == buttons.length - 1)
-                x_alignment = St.Align.END;
-            else
-                x_alignment = St.Align.MIDDLE;
-
-            if (wantsfocus) {
-                this._initialKeyFocus = buttonInfo.button;
-                focusSetExplicitly = true;
-            }
-
-            if (!focusSetExplicitly && !nofocus && (this._initialKeyFocus == this._dialogLayout ||
-                this._buttonLayout.contains(this._initialKeyFocus)))
-            {
-                this._initialKeyFocus = buttonInfo.button;
-            }
-            this._buttonLayout.add(buttonInfo.button,
-                                   { expand: true,
-                                     x_fill: false,
-                                     y_fill: false,
-                                     x_align: x_alignment,
-                                     y_align: St.Align.MIDDLE });
-
-            buttonInfo.button.connect('clicked', action);
-
-            if (key)
-                this._actionKeys[key] = action;
+        for (let buttonInfo of buttons) {
+            this.addButton(buttonInfo);
         }
+    }
 
-        // Fade in buttons if there weren't any before
-        if (!hadChildren && buttons.length > 0) {
-            this._buttonLayout.opacity = 0;
-            Tweener.addTween(this._buttonLayout,
-                             { opacity: 255,
-                               time: FADE_IN_BUTTONS_TIME,
-                               transition: 'easeOutQuad',
-                               onComplete: Lang.bind(this, function() {
-                                   this.emit('buttons-set');
-                               })
-                             });
-        } else {
-            this.emit('buttons-set');
-        }
+    addButton(buttonInfo) {
+        return this.dialogLayout.addButton(buttonInfo);
+    }
 
-    },
+    _fadeOpen() {
+        this._monitorConstraint.index = global.display.get_current_monitor();
 
-    _onKeyPressEvent: function(object, keyPressEvent) {
-        let modifiers = Cinnamon.get_event_state(keyPressEvent);
-        let ctrlAltMask = Clutter.ModifierType.CONTROL_MASK | Clutter.ModifierType.MOD1_MASK;
-        let symbol = keyPressEvent.get_key_symbol();
+        this._setState(State.OPENING);
 
-        let action = this._actionKeys[symbol];
-
-        if (action) {
-            action();
-            return;
-        }
-
-        if (symbol === Clutter.KEY_Escape && !(modifiers & ctrlAltMask)) {
-            this.close();
-            return;
-        }
-    },
-
-    _onGroupDestroy: function() {
-        this.emit('destroy');
-    },
-
-    _fadeOpen: function() {
-        let monitor = Main.layoutManager.currentMonitor;
-
-        this._backgroundBin.set_position(monitor.x, monitor.y);
-        this._backgroundBin.set_size(monitor.width, monitor.height);
-
-        this.state = State.OPENING;
-
-        this._dialogLayout.opacity = 255;
+        this.dialogLayout.opacity = 255;
         if (this._lightbox)
             this._lightbox.show();
-        this._group.opacity = 0;
-        this._group.show();
-        Tweener.addTween(this._group,
-                         { opacity: 255,
-                           time: OPEN_AND_CLOSE_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this,
-                               function() {
-                                   this.state = State.OPENED;
-                                   this.emit('opened');
-                               })
-                         });
-    },
+        this.opacity = 0;
+        this.show();
+        this.ease({
+            opacity: 255,
+            duration: this.openAndCloseTime,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => {
+                this._setState(State.OPENED);
+                this.emit('opened');
+            }
+        });
+    }
+
+    setInitialKeyFocus(actor) {
+        if (this._initialKeyFocusDestroyId)
+            this._initialKeyFocus.disconnect(this._initialKeyFocusDestroyId);
 
-    setInitialKeyFocus: function(actor) {
         this._initialKeyFocus = actor;
-    },
+
+        this._initialKeyFocusDestroyId = actor.connect('destroy', () => {
+            this._initialKeyFocus = null;
+            this._initialKeyFocusDestroyId = 0;
+        });
+    }
 
     /**
      * open:
@@ -300,7 +231,7 @@ ModalDialog.prototype = {
      *
      * Opens and displays the modal dialog.
      */
-    open: function(timestamp) {
+    open(timestamp) {
         if (this.state == State.OPENED || this.state == State.OPENING)
             return true;
 
@@ -309,7 +240,7 @@ ModalDialog.prototype = {
 
         this._fadeOpen();
         return true;
-    },
+    }
 
     /**
      * close:
@@ -318,25 +249,28 @@ ModalDialog.prototype = {
      *
      * Closes the modal dialog.
      */
-    close: function(timestamp) {
+    close(timestamp) {
         if (this.state == State.CLOSED || this.state == State.CLOSING)
             return;
 
-        this.state = State.CLOSING;
+        this._setState(State.CLOSING);
         this.popModal(timestamp);
         this._savedKeyFocus = null;
 
-        Tweener.addTween(this._group,
-                         { opacity: 0,
-                           time: OPEN_AND_CLOSE_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this,
-                               function() {
-                                   this.state = State.CLOSED;
-                                   this._group.hide();
-                               })
-                         });
-    },
+        this.ease({
+            opacity: 0,
+            duration: this.openAndCloseTime,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => {
+                this._setState(State.CLOSED);
+                this.hide();
+                this.emit('closed');
+
+                if (this._destroyOnClose)
+                    this.destroy();
+            }
+        });
+    }
 
     /**
      * popModal:
@@ -347,16 +281,16 @@ ModalDialog.prototype = {
      * dialog insensitive as well, so it needs to be followed shortly
      * by either a %close() or a %pushModal()
      */
-    popModal: function(timestamp) {
+    popModal(timestamp) {
         if (!this._hasModal)
             return;
 
         let focus = global.stage.key_focus;
-        if (focus && this._group.contains(focus))
+        if (focus && this.contains(focus))
             this._savedKeyFocus = focus;
         else
             this._savedKeyFocus = null;
-        Main.popModal(this._group, timestamp);
+        Main.popModal(this, timestamp);
 
         if (!Meta.is_wayland_compositor()) {
             Gdk.Display.get_default().sync();
@@ -366,7 +300,7 @@ ModalDialog.prototype = {
 
         if (!this._cinnamonReactive)
             this._eventBlocker.raise_top();
-    },
+    }
 
     /**
      * pushModal:
@@ -376,23 +310,25 @@ ModalDialog.prototype = {
      * Pushes the modal to the modal stack so that it grabs the required
      * inputs.
      */
-    pushModal: function (timestamp) {
+    pushModal(timestamp) {
         if (this._hasModal)
             return true;
-        if (!Main.pushModal(this._group, timestamp))
+        if (!Main.pushModal(this, timestamp))
             return false;
 
         this._hasModal = true;
         if (this._savedKeyFocus) {
             this._savedKeyFocus.grab_key_focus();
             this._savedKeyFocus = null;
-        } else
-            this._initialKeyFocus.grab_key_focus();
+        } else {
+            let focus = this._initialKeyFocus || this.dialogLayout.initialKeyFocus;
+            focus.grab_key_focus();
+        }
 
         if (!this._cinnamonReactive)
             this._eventBlocker.lower_bottom();
         return true;
-    },
+    }
 
     /**
      * _fadeOutDialog:
@@ -412,7 +348,7 @@ ModalDialog.prototype = {
      * immediately, but the lightbox should remain until the logout is
      * complete.
      */
-    _fadeOutDialog: function(timestamp) {
+     _fadeOutDialog(timestamp) {
         if (this.state == State.CLOSED || this.state == State.CLOSING)
             return;
 
@@ -420,18 +356,16 @@ ModalDialog.prototype = {
             return;
 
         this.popModal(timestamp);
-        Tweener.addTween(this._dialogLayout,
-                         { opacity: 0,
-                           time:    FADE_OUT_DIALOG_TIME,
-                           transition: 'easeOutQuad',
-                           onComplete: Lang.bind(this,
-                               function() {
-                                   this.state = State.FADED_OUT;
-                               })
-                         });
+        this.dialogLayout.ease({
+            opacity: 0,
+            duration: FADE_OUT_DIALOG_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => {
+                this._setState(State.FADED_OUT);
+            }
+        });
     }
-};
-Signals.addSignalMethods(ModalDialog.prototype);
+});
 
 /**
  * #ConfirmDialog
@@ -443,43 +377,42 @@ Signals.addSignalMethods(ModalDialog.prototype);
  *
  * Inherits: ModalDialog.ModalDialog
  */
-function ConfirmDialog(label, callback){
-    this._init(label, callback);
-}
-
-ConfirmDialog.prototype = {
-    __proto__: ModalDialog.prototype,
+var ConfirmDialog = GObject.registerClass(
+class ConfirmDialog extends ModalDialog {
 
     /**
      * _init:
-     * @label (string): label to display on the confirm dialog
+     * @description (string): label to display on the confirm dialog
      * @callback (function): function to call when user clicks "yes"
      *
      * Constructor function.
      */
-    _init: function(label, callback){
-        ModalDialog.prototype._init.call(this);
-        this.contentLayout.add(new St.Label({ text:        _("Confirm"),
-                                              style_class: 'confirm-dialog-title',
-                                              important:   true }));
-        this.contentLayout.add(new St.Label({text: label}));
+     _init(description, callback) {
+        super._init();
+
+        let title = _("Confirm");
+
+        let content = new Dialog.MessageDialogContent({ title, description });
+        this.contentLayout.add_child(content);
         this.callback = callback;
 
         this.setButtons([
             {
                 label: _("No"),
-                action: Lang.bind(this, this.destroy)
+                action: this.destroy.bind(this),
+                key: Clutter.KEY_Escape,
             },
             {
                 label: _("Yes"),
-                action: Lang.bind(this, function(){
+                action: () => {
                     this.destroy();
                     this.callback();
-                })
+                },
+                destructive_action: true
             }
         ]);
-    },
-};
+    }
+});
 
 /**
  * #NotifyDialog
@@ -491,31 +424,32 @@ ConfirmDialog.prototype = {
  *
  * Inherits: ModalDialog.ModalDialog
  */
-function NotifyDialog(label){
-    this._init(label);
-}
-
-NotifyDialog.prototype = {
-    __proto__: ModalDialog.prototype,
+var NotifyDialog = GObject.registerClass(
+class NotifyDialog extends ModalDialog {
 
     /**
      * _init:
-     * @label (string): label to display on the notify dialog
+     * @description (string): label to display on the notify dialog
      *
      * Constructor function.
      */
-    _init: function(label){
-        ModalDialog.prototype._init.call(this);
-        this.contentLayout.add(new St.Label({text: label}));
+    _init(description) {
+        super._init();
+
+        let title = _("Attention");
+
+        let content = new Dialog.MessageDialogContent({ title, description });
+        this.contentLayout.add_child(content);
 
         this.setButtons([
             {
                 label: _("OK"),
-                action: Lang.bind(this, this.destroy)
+                action: this.destroy.bind(this),
+                default: true,
             }
         ]);
-    },
-};
+    }
+});
 
 /**
  * #InfoOSD
@@ -531,11 +465,7 @@ NotifyDialog.prototype = {
  * destroying it after usage (via the %destroy function), or hiding it with
  * %hide for later reuse.
  */
-function InfoOSD(text) {
-    this._init(text);
-}
-
-InfoOSD.prototype = {
+var InfoOSD = class {
 
     /**
      * _init:
@@ -544,14 +474,22 @@ InfoOSD.prototype = {
      * Constructor function. Creates an OSD and adds it to the chrome. Adds a
      * label with text @text if specified.
      */
-    _init: function(text) {
-        this.actor = new St.BoxLayout({vertical: true, style_class: "info-osd", important: true});
+    constructor(text) {
+        this.actor = new St.BoxLayout({
+            vertical: true,
+            style_class: "info-osd",
+            important: true
+        });
+
         if (text) {
             let label = new St.Label({text: text});
             this.actor.add(label);
         }
-        Main.layoutManager.addChrome(this.actor, {visibleInFullscreen: false, affectsInputRegion: false});
-    },
+        Main.layoutManager.addChrome(this.actor, {
+            visibleInFullscreen: false,
+            affectsInputRegion: false
+        });
+    }
 
     /**
      * show:
@@ -561,7 +499,7 @@ InfoOSD.prototype = {
      * Shows the OSD at the center of monitor @monitorIndex. Shows at the
      * primary monitor if not specified.
      */
-    show: function(monitorIndex) {
+    show(monitorIndex) {
         let monitor;
 
         if (!monitorIndex) {
@@ -579,26 +517,26 @@ InfoOSD.prototype = {
 
         this.actor.set_position(x, y);
         this.actor.opacity = 255;
-    },
+    }
 
     /**
      * hide:
      *
      * Hides the OSD.
      */
-    hide: function() {
+    hide() {
         this.actor.hide();
-    },
+    }
 
     /**
      * destroy:
      *
      * Destroys the OSD
      */
-    destroy: function() {
+    destroy() {
         this.hide();
         Main.layoutManager.removeChrome(this.actor);
-    },
+    }
 
     /**
      * addText:
@@ -607,10 +545,10 @@ InfoOSD.prototype = {
      *
      * Adds a text label displaying @text to the OSD
      */
-    addText: function(text, params) {
+    addText(text, params) {
         let label = new St.Label({text: text});
         this.actor.add(label, params);
-    },
+    }
 
     /**
      * addActor:
@@ -619,7 +557,7 @@ InfoOSD.prototype = {
      *
      * Adds the actor @actor to the OSD
      */
-    addActor: function(actor, params) {
+    addActor(actor, params) {
         this.actor.add(actor, params);
     }
-}
+};
