@@ -5,9 +5,6 @@
 #include "cinnamon-app.h"
 #include "cinnamon-app-system.h"
 
-#define SN_API_NOT_YET_FROZEN 1
-#include <libsn/sn.h>
-
 G_BEGIN_DECLS
 
 CinnamonApp* _cinnamon_app_new_for_window (MetaWindow *window);
@@ -16,7 +13,7 @@ CinnamonApp* _cinnamon_app_new (GMenuTreeEntry *entry);
 
 void _cinnamon_app_set_entry (CinnamonApp *app, GMenuTreeEntry *entry);
 
-void _cinnamon_app_handle_startup_sequence (CinnamonApp *app, SnStartupSequence *sequence);
+void _cinnamon_app_handle_startup_sequence (CinnamonApp *app, MetaStartupSequence *sequence);
 
 void _cinnamon_app_add_window (CinnamonApp *app, MetaWindow *window);
 
