@@ -37,18 +37,23 @@ G_BEGIN_DECLS
 typedef enum {
   ST_ALIGN_START,
   ST_ALIGN_MIDDLE,
-  ST_ALIGN_END,
-  ST_ALIGN_CENTER_SPECIAL
+  ST_ALIGN_END
 } StAlign;
 
 typedef enum {
   ST_ICON_SYMBOLIC,
   ST_ICON_FULLCOLOR,
   ST_ICON_APPLICATION,
-  ST_ICON_DOCUMENT,
-  ST_ICON_FADED
+  ST_ICON_DOCUMENT
 } StIconType;
 
+typedef enum {
+  ST_BACKGROUND_SIZE_AUTO,
+  ST_BACKGROUND_SIZE_CONTAIN,
+  ST_BACKGROUND_SIZE_COVER,
+  ST_BACKGROUND_SIZE_FIXED
+} StBackgroundSize;
+
 G_END_DECLS
 
 #endif /* __ST_TYPES_H__ */
